require 'rest-client'
require 'json'
require 'base64'
require 'uri'

# Errors
require_relative 'bills-payment/errors/bills_payment_error'
require_relative 'bills-payment/errors/authentication_error'
require_relative 'bills-payment/errors/invalid_request_error'
require_relative 'bills-payment/errors/connection_error'

# Actions
require_relative 'bills-payment/actions/post'

# Resources
require_relative 'bills-payment/bills_payment_object'
require_relative 'bills-payment/worker'

module BillsPayment
  @base_url = 'http://14.98.171.134:9090/'

  class << self
    # BillsPaymentError :api_key, :base_url

    def request(api_url, method, params = {})
      # raise(AuthenticationError, 'Set your API Key using BillsPayment.api_key = <API_KEY>') unless api_key
      begin
        url = URI.join(@base_url, api_url).to_s
        # response = RestClient::Request.execute(method: method, url: url, payload: params.to_json)#, headers: request_headers)
        
        RestClient.post url, params.to_json

        JSON.parse(response) unless response.empty?
      rescue RestClient::ExceptionWithResponse => e
        if (response_code = e.http_code) && (response_body = e.http_body)
          handle_api_error(response_code, JSON.parse(response_body))
        else
          handle_restclient_error(e)
        end
      rescue RestClient::Exception, Errno::ECONNREFUSED => e
        handle_restclient_error(e)
      end
    end

    private

    def request_headers
      {
        Authorization: "Basic #{encoded_api_key}",
        content_type: :json,
        accept: :json
      }
    end

    def encoded_api_key
      @encoded_api_key ||= Base64.urlsafe_encode64(api_key)
    end

    def handle_api_error(code, body)
      case code
      when 400, 404
        raise InvalidRequestError, body['message']
      when 401
        raise AuthenticationError, body['message']
      else
        raise BillsPaymentError, body['message']
      end
    end

    def handle_restclient_error(exception)
      message =
        case exception
        when RestClient::RequestTimeout
          'Could not connect to BillsPayment. Check your internet connection and try again.'
        when RestClient::ServerBrokeConnection
          'The connetion with BillsPayment terminated before the request completed. Please try again.'
        else
          'There was a problem connection with BillsPayment. Please try again. If the problem persists contact contact@BillsPayment.com'
        end

      raise ConnectionError, message
    end
  end
end

